"""
LLM wrapper for OpenAI chat model.
"""
import promptlayer  # Do not remove this line.
from langchain.callbacks import PromptLayerCallbackHandler
from langchain.chat_models import ChatOpenAI

from llm_factory.llm_base import LLMBase


class LLMOpenAIChat(LLMBase):
    """
    LLM wrapper for OpenAI chat model.
    """

    def _validate_args(self, **kwargs):
        assert "model_name" in kwargs, "'model_name' must be provided"
        assert "temperature" in kwargs, "'temperature' must be provided"
        assert "max_tokens" in kwargs, "'max_tokens' must be provided"
        if "verbose" in kwargs:
            assert isinstance(kwargs["verbose"],
                              bool), "'verbose' must be a boolean"
        if "pl_tags" in kwargs:
            assert isinstance(kwargs["pl_tags"],
                              list), "'pl_tags' must be a list"
            for tag in kwargs["pl_tags"]:
                assert isinstance(
                    tag, str), "'pl_tags' must be a list of strings"

    def _build_llm(self, **kwargs):
        if "pl_tags" in kwargs:
            llm = ChatOpenAI(
                model=kwargs["model_name"],
                temperature=kwargs["temperature"],
                max_tokens=kwargs["max_tokens"],
                verbose=kwargs.get("verbose", False),
                callbacks=[PromptLayerCallbackHandler(
                    pl_tags=kwargs.get("pl_tags", []))
                ],
            )
        else:
            llm = ChatOpenAI(
                model=kwargs["model_name"],
                temperature=kwargs["temperature"],
                max_tokens=kwargs["max_tokens"],
                verbose=kwargs.get("verbose", False),
            )
        return llm

    @staticmethod
    def get_default_args():
        return {
            "model_name": "gpt-4",
            "temperature": 0,
            "max_tokens": 256,
        }



"""
Test LLMFactory

Usage:
    pytest src/tests/test_llm_factory.py -v --log-cli-level INFO
"""
import sys
from pathlib import Path
sys.path.insert(0, str(Path(__file__).parent.parent))

from llm_factory.llm_factory import LLMFactory
from langchain.chat_models import ChatOpenAI
import pytest
from dotenv import load_dotenv


load_dotenv(override=True)


@pytest.fixture
def llm_factory():
    return LLMFactory()


def test_build_llm(llm_factory: LLMFactory):
    llm = llm_factory.build_llm(
        LLMFactory.LLM_OPENAI_CHAT_NAME,
        model_name="model_name",
        temperature=0.5,
        max_tokens=10,
        pl_tags=["test"],
    )
    assert isinstance(llm, ChatOpenAI)


def test_build_llm_default(llm_factory: LLMFactory):
    llm = llm_factory.build_llm_default()
    assert isinstance(llm, ChatOpenAI)



from abc import ABC, abstractmethod


class LLMBase(ABC):
    @abstractmethod
    def _validate_args(self, **kwargs):
        pass

    @abstractmethod
    def _build_llm(self, **kwargs):
        pass

    def get_llm(self, **kwargs):
        """
        Returns a LLM instance.
        """
        self._validate_args(**kwargs)
        return self._build_llm(**kwargs)
